@@ -1,4 +1,5 @@
 #include <gnome.h>
+#include <mikmod.h>
 
 #include "sge_core.h"
 #include "board_engine.h"
@@ -26,8 +27,9 @@ typedef struct s_alignment {
 	gint length;
 } T_Alignment;
 
+SAMPLE *swap_sfx;
+
 gint gi_score, gi_current_score, gi_game_running;
-gboolean gweled_in_timer_mode = TRUE;
 
 gint gi_total_gems_removed;
 gint gi_gems_removed_per_move;
@@ -36,6 +38,7 @@ gint gi_bonus_multiply;
 gint gi_previous_bonus_at;
 gint gi_next_bonus_at;
 gint gi_trigger_bonus;
+guint g_steps_for_timer;
 
 gint gi_gem_clicked = 0;
 gint gi_x_click = 0;
@@ -64,6 +67,8 @@ extern GtkWidget *g_bonus_label;
 extern gint gi_gems_pixbuf[7];
 extern gint gi_cursor_pixbuf;
 
+extern GweledPrefs prefs;
+
 gchar
 get_new_tile (void)
 {
@@ -110,8 +115,9 @@ gweled_start_new_game (void)
 	gi_previous_bonus_at = 0;
 	gi_next_bonus_at = FIRST_BONUS_AT;
 	gi_trigger_bonus = 0;
+	g_steps_for_timer = FIRST_BONUS_AT / TOTAL_STEPS_FOR_TIMER;
 
-	if(gweled_in_timer_mode)
+	if(prefs.timer_mode)
 		gi_total_gems_removed = FIRST_BONUS_AT / 2;
 	else
 		gi_total_gems_removed = 0;
@@ -120,20 +126,18 @@ gweled_start_new_game (void)
 	gtk_label_set_markup ((GtkLabel *) g_score_label, "<span weight=\"bold\">000000</span>");
 	gtk_label_set_markup ((GtkLabel *) g_bonus_label, "<span weight=\"bold\">x1</span>");
 
-	memset (gi_nb_of_tiles, 0, 8 * sizeof (int));
+	memset (gi_nb_of_tiles, 0, 7 * sizeof (int));
 
 	for (i = 0; i < BOARD_WIDTH; i++)
-		for (j = 0; j < BOARD_HEIGHT; j++) {
+		for (j = 0; j < BOARD_HEIGHT; j++)
+		{
 			gpc_game_board[i][j] = get_new_tile ();
 			gi_nb_of_tiles[gpc_game_board[i][j]]++;
 			g_gem_objects[i][j] =
-			    sge_create_object (i * gi_tile_width,
-					       (j -
-						BOARD_HEIGHT) *
-					       gi_tile_height, 1,
-					       gi_gems_pixbuf
-					       [gpc_game_board[i][j]]);
-			
+			    sge_create_object (i * prefs.tile_width,
+							(j - BOARD_HEIGHT) * prefs.tile_height,
+							1,
+							gi_gems_pixbuf[gpc_game_board[i][j]]);
 		}
 
 	g_do_not_score = TRUE;
@@ -155,7 +159,7 @@ gpc_game_board[5][7] = 1;
 
 	for (i = 0; i < BOARD_WIDTH; i++)
 		for (j = 0; j < BOARD_HEIGHT; j++)
-			g_gem_objects[i][j] = sge_create_object (i * gi_tile_width, (j - BOARD_HEIGHT) * gi_tile_height, 1,
+			g_gem_objects[i][j] = sge_create_object (i * prefs.tile_width, (j - BOARD_HEIGHT) * prefs.tile_height, 1,
 													 gi_gems_pixbuf[gpc_game_board[i][j]]);
 			
 	gweled_gems_fall_into_place ();
@@ -261,8 +265,10 @@ gweled_refill_board (void)
 
 	for (i = 0; i < BOARD_WIDTH; i++)
 		for (j = 0; j < BOARD_HEIGHT; j++)
-			if (gpc_game_board[i][j] == -1) {
-				for (k = j; k > 0; k--) {
+			if (gpc_game_board[i][j] == -1)
+			{
+				for (k = j; k > 0; k--)
+				{
 					gpc_game_board[i][k] =
 					    gpc_game_board[i][k - 1];
 					g_gem_objects[i][k] =
@@ -273,25 +279,15 @@ gweled_refill_board (void)
 
 				// make sure the new tile appears outside of the screen (1st row is special-cased)
 				if (j && g_gem_objects[i][1])
-					g_gem_objects[i][0] =
-					    sge_create_object (i *
-							       gi_tile_width,
-							       g_gem_objects
-							       [i][1]->y -
-							       gi_tile_height,
-							       1,
-							       gi_gems_pixbuf
-							       [gpc_game_board
-								[i][0]]);
+					g_gem_objects[i][0] = sge_create_object (i * prefs.tile_width,
+											g_gem_objects[i][1]->y - prefs.tile_height,
+											1,
+											gi_gems_pixbuf[gpc_game_board[i][0]]);
 				else
-					g_gem_objects[i][0] =
-					    sge_create_object (i *
-							       gi_tile_width,
-							       -gi_tile_height,
-							       1,
-							       gi_gems_pixbuf
-							       [gpc_game_board
-								[i][0]]);
+					g_gem_objects[i][0] = sge_create_object (i * prefs.tile_width,
+											-prefs.tile_height,
+											1,
+											gi_gems_pixbuf[gpc_game_board[i][0]]);
 			}
 }
 
@@ -308,8 +304,8 @@ delete_alignment_from_board (gpointer alignment_pointer, gpointer user_data)
 // delete alignment
 	if (alignment->direction == 1)	// horizontal
 	{
-		xhotspot = (alignment->x * gi_tile_width + alignment->length * gi_tile_width / 2);
-		yhotspot = (alignment->y * gi_tile_height + gi_tile_height / 2);
+		xhotspot = (alignment->x * prefs.tile_width + alignment->length * prefs.tile_width / 2);
+		yhotspot = (alignment->y * prefs.tile_height + prefs.tile_height / 2);
 		for (i = alignment->x; i < alignment->x + alignment->length; i++) {
 			if (gpc_game_board[i][alignment->y] != -1) {
 				gi_gems_removed_per_move++;
@@ -318,8 +314,8 @@ delete_alignment_from_board (gpointer alignment_pointer, gpointer user_data)
 			}
 		}
 	} else {
-		xhotspot = (alignment->x * gi_tile_width + gi_tile_width / 2);
-		yhotspot = (alignment->y * gi_tile_height + alignment->length * gi_tile_height / 2);
+		xhotspot = (alignment->x * prefs.tile_width + prefs.tile_width / 2);
+		yhotspot = (alignment->y * prefs.tile_height + alignment->length * prefs.tile_height / 2);
 		for (i = alignment->y; i < alignment->y + alignment->length; i++) {
 			if (gpc_game_board[alignment->x][i] != -1) {
 				gi_gems_removed_per_move++;
@@ -513,7 +509,7 @@ board_engine_loop (gpointer data)
 	static gint x1, y1, x2, y2, time_slice = 0;
 	static T_SGEObject *cursor[2] = { NULL, NULL };
 	gchar msg_buffer[200];
-	static guint steps_for_timer = FIRST_BONUS_AT / TOTAL_STEPS_FOR_TIMER;
+	gint hiscore_rank;
 
 	time_slice++;
 	
@@ -526,13 +522,16 @@ board_engine_loop (gpointer data)
 	}
 
 /* Let's first check if we are in timer mode, and penalize the player if necessary */
-	if (gweled_in_timer_mode && gi_game_running && (time_slice % 10 == 0))
+	if (prefs.timer_mode && gi_game_running && (time_slice % 10 == 0))
 	{
-		gi_total_gems_removed -= steps_for_timer;
+		gi_total_gems_removed -= g_steps_for_timer;
 		if (gi_total_gems_removed <= gi_previous_bonus_at) {
 			gweled_draw_message ("time's up #");
 			gi_game_running = 0;
-			gnome_score_log ((gfloat) gi_score, "timed", TRUE);
+ 			hiscore_rank = gnome_score_log ((gfloat) gi_score, "timed", TRUE);
+ 			if (hiscore_rank > 0)
+ 				show_hiscores (hiscore_rank);
+ 			show_hiscores (gi_score);
 			g_do_not_score = FALSE;
 			gi_state = _IDLE;
 		} else
@@ -550,8 +549,8 @@ board_engine_loop (gpointer data)
 			gi_state = _FIRST_GEM_CLICKED;
 			if (cursor[0])
 				sge_destroy_object (cursor[0], NULL);
-			cursor[0] = sge_create_object (gi_tile_width * x1,
-					gi_tile_height * y1,
+			cursor[0] = sge_create_object (prefs.tile_width * x1,
+					prefs.tile_height * y1,
 					2, gi_cursor_pixbuf);
 			gi_gem_clicked = 0;
 			gi_gem_dragged = 0;
@@ -568,15 +567,17 @@ board_engine_loop (gpointer data)
 				// If the player clicks an adjacent gem, try to swap
 				if (cursor[1])
 					sge_destroy_object (cursor[1], NULL);
-				cursor[1] = sge_create_object (gi_tile_width * x2,
-						gi_tile_height * y2,
+				cursor[1] = sge_create_object (prefs.tile_width * x2,
+						prefs.tile_height * y2,
 						2, gi_cursor_pixbuf);
 				sge_object_move_to (g_gem_objects[x1][y1],
-						x2 * gi_tile_width,
-						y2 * gi_tile_height);
+						x2 * prefs.tile_width,
+						y2 * prefs.tile_height);
 				sge_object_move_to (g_gem_objects[x2][y2],
-						x1 * gi_tile_width,
-						y1 * gi_tile_height);
+						x1 * prefs.tile_width,
+						y1 * prefs.tile_height);
+				if(swap_sfx)
+					Sample_Play(swap_sfx, 0, 0);						
 				gi_state = _SECOND_GEM_CLICKED;
 			} else if((x1 == x2) && (y1 == y2)) {
 				if (cursor[1])
@@ -596,8 +597,8 @@ board_engine_loop (gpointer data)
 				y1 = y2;
 				if (cursor[0])
 					sge_destroy_object (cursor[0], NULL);
-				cursor[0] = sge_create_object (gi_tile_width * x1,
-						gi_tile_height * y1,
+				cursor[0] = sge_create_object (prefs.tile_width * x1,
+						prefs.tile_height * y1,
 						2, gi_cursor_pixbuf);
 			}
 		}else if(gi_gem_dragged)
@@ -605,8 +606,8 @@ board_engine_loop (gpointer data)
 			//printf("gem dragged\n");
 			if (cursor[1])
 					sge_destroy_object (cursor[1], NULL);
-				cursor[1] = sge_create_object (gi_tile_width * gi_x_drag,
-						gi_tile_height * gi_y_drag,
+				cursor[1] = sge_create_object (prefs.tile_width * gi_x_drag,
+						prefs.tile_height * gi_y_drag,
 						2, gi_cursor_pixbuf);
 		}
 		break;
@@ -617,11 +618,11 @@ board_engine_loop (gpointer data)
 			if (!gweled_is_part_of_an_alignment (x1, y1) && !gweled_is_part_of_an_alignment (x2, y2)) {
 				//gweled_draw_game_message("illegal move !", 1.0);
 				sge_object_move_to (g_gem_objects[x1][y1],
-						x2 * gi_tile_width,
-						y2 * gi_tile_height);
+						x2 * prefs.tile_width,
+						y2 * prefs.tile_height);
 				sge_object_move_to (g_gem_objects[x2][y2],
-						x1 * gi_tile_width,
-						y1 * gi_tile_height);
+						x1 * prefs.tile_width,
+						y1 * prefs.tile_height);
 				gi_state = _ILLEGAL_MOVE;
 			} else {
 				if (cursor[0])
@@ -666,11 +667,11 @@ board_engine_loop (gpointer data)
 			gi_state = _BOARD_REFILLING;
 		} else {
 			if (gweled_check_for_moves_left (NULL, NULL) == FALSE) {
-				if ((gi_next_bonus_at == FIRST_BONUS_AT) || (gweled_in_timer_mode)) {
+				if ((gi_next_bonus_at == FIRST_BONUS_AT) || (prefs.timer_mode)) {
 					gint i, j;
 
 					gweled_draw_game_message ("no moves left #", 1.0);
-					memset (gi_nb_of_tiles, 0, 8 * sizeof (int));
+					memset (gi_nb_of_tiles, 0, 7 * sizeof (int));
 
 					for (i = 0; i < BOARD_WIDTH; i++)
 						for (j = 0; j < BOARD_HEIGHT; j++) {
@@ -687,8 +688,8 @@ board_engine_loop (gpointer data)
 					for (i = 0; i < BOARD_WIDTH; i++)
 						for (j = 0; j < BOARD_HEIGHT; j++) {
 							g_gem_objects[i][j] = sge_create_object
-								(i * gi_tile_width,
-							    	(j - BOARD_HEIGHT) * gi_tile_height,
+								(i * prefs.tile_width,
+							    	(j - BOARD_HEIGHT) * prefs.tile_height,
 								1, gi_gems_pixbuf[gpc_game_board[i][j]]);
 						}
 					gweled_gems_fall_into_place ();
@@ -696,7 +697,9 @@ board_engine_loop (gpointer data)
 				} else {
 					gweled_draw_message ("no moves left #");
 					gi_game_running = 0;
-					gnome_score_log ((gfloat) gi_score, "easy", TRUE);
+					hiscore_rank = gnome_score_log ((gfloat) gi_score, "easy", TRUE);
+ 					if (hiscore_rank > 0)
+ 						show_hiscores (hiscore_rank);
 					g_do_not_score = FALSE;
 					gi_state = _IDLE;
 				}
@@ -712,15 +715,15 @@ board_engine_loop (gpointer data)
 			if (gi_total_gems_removed >= gi_next_bonus_at) {
 				gi_previous_bonus_at = gi_next_bonus_at;
 				gi_next_bonus_at *= 2;
-				if (gweled_in_timer_mode)
-					steps_for_timer = (gi_next_bonus_at - gi_previous_bonus_at) / TOTAL_STEPS_FOR_TIMER + 1;
+				if (prefs.timer_mode)
+					g_steps_for_timer = (gi_next_bonus_at - gi_previous_bonus_at) / TOTAL_STEPS_FOR_TIMER + 1;
 				gi_bonus_multiply++;
 				sprintf (msg_buffer, "bonus x%d", gi_bonus_multiply >> 1);
 				gweled_draw_game_message (msg_buffer, 1.0);
 				gweled_delete_gems_for_bonus ();
 				gweled_take_down_deleted_gems ();
 				gweled_remove_gems_and_update_score ();
-				if (gweled_in_timer_mode)
+				if (prefs.timer_mode)
 					gi_total_gems_removed = (gi_next_bonus_at + gi_previous_bonus_at) / 2;
 				gtk_progress_bar_set_fraction ((GtkProgressBar *) g_progress_bar,
 					(float) (gi_total_gems_removed - gi_previous_bonus_at) /
