@@ -1,4 +1,13 @@
-// EXPORTS
+#ifndef _BOARD_ENGINE_H_
+#define _BOARD_ENGINE_H_
+
+typedef struct s_gweled_prefs
+{
+	gboolean timer_mode;
+	gint tile_width;
+	gint tile_height;
+}GweledPrefs;
+
 extern gint gi_game_running;
 extern gboolean gweled_in_timer_mode;
 
@@ -18,3 +27,5 @@ void gweled_refill_board(void);
 gboolean gweled_check_for_moves_left(int *pi, int *pj);
 
 gboolean board_engine_loop(gpointer data);
+
+#endif
