@@ -4,6 +4,7 @@
 
 #include <gnome.h>
 #include <glade/glade.h>
+#include <mikmod.h>
 
 #include "callbacks.h"
 
@@ -31,6 +32,10 @@ extern gint gi_gem_dragged;
 extern gint gi_x_drag;
 extern gint gi_y_drag;
 
+extern GweledPrefs prefs;
+
+SAMPLE *click_sfx;
+
 static gint gi_dragging = 0;
 
 void
@@ -63,65 +68,8 @@ on_new1_activate (GtkMenuItem * menuitem, gpointer user_data)
 void
 on_scores1_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
-	GtkWidget *box;
-	GtkWidget *label;
-	char **names;
-	gfloat *scores;
-	time_t *scoretimes;
-	int i, nb_scores;
-	char *buffer;
-
-	nb_scores = gnome_score_get_notable ("gweled", "easy", &names, &scores, &scoretimes);
-
-// FIXME: that's a temporary solution to display the timed game highscores
-    if (gweled_in_timer_mode)
-	   	nb_scores = gnome_score_get_notable ("gweled", "timed", &names, &scores, &scoretimes);
-	else
-	   	nb_scores = gnome_score_get_notable ("gweled", "easy", &names, &scores, &scoretimes);
-	   	
-	if (nb_scores) {
-		for (i = 0; i < 10; i++) {
-			buffer = g_strdup_printf ("nameLabel%02d", i + 1);
-			label = glade_xml_get_widget(gweled_xml, buffer);
-			if (label)
-				if(i < nb_scores) {
-					gtk_label_set_text (GTK_LABEL (label), names[i]);
-					g_free (names[i]);
-				} else {
-					gtk_label_set_text (GTK_LABEL (label), "");
-				}
-			g_free (buffer);
-
-			buffer = g_strdup_printf ("scoreLabel%02d", i + 1);
-			label = glade_xml_get_widget(gweled_xml, buffer);
-			g_free (buffer);
-			if (label)
-				if(i < nb_scores) {
-					buffer = g_strdup_printf ("%.0f", scores[i]);
-					gtk_label_set_text (GTK_LABEL (label), buffer);
-					g_free (buffer);
-				} else {
-					gtk_label_set_text (GTK_LABEL (label), "");
-				}
-		}
-
-		for (; i < nb_scores; i++)
-			g_free (names[i]);
-
-		g_free (names);
-		g_free (scores);
-		g_free (scoretimes);
-
-		gtk_widget_show (g_score_window);
-	} else {
-		box = gtk_message_dialog_new (GTK_WINDOW (g_main_window),
-					      GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_MESSAGE_INFO,
-					      GTK_BUTTONS_OK,
-					      _("No highscores recorded yet"));
-		gtk_dialog_run (GTK_DIALOG (box));
-		gtk_widget_destroy (box);
-	}
+	// See comments in show_hiscores() for meaning of 0 as argument
+ 	show_hiscores (0);
 }
 
 void
@@ -154,7 +102,7 @@ on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
 	pixbuf = gdk_pixbuf_new_from_file (PACKAGE_DATA_DIR "/gweled/gweled_logo.png", NULL);
 
 	about = gnome_about_new(_("Gweled"), VERSION,
-			"Copyright \xc2\xa9 2003-2004 Sebastien Delestaing",
+			"Copyright \xc2\xa9 2003-2005 Sebastien Delestaing",
 			_("This is a GNOME port of the PalmOS/Windows/Java game \"Bejeweled\" (aka \"Diamond Mine\")."),
 			(const char **)authors,
 			(const char **)documenters,
@@ -195,13 +143,17 @@ drawing_area_button_event_cb (GtkWidget * widget, GdkEventButton * event, gpoint
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-		x_press = event->x / gi_tile_width;
-		y_press = event->y / gi_tile_height;
+		x_press = event->x / prefs.tile_width;
+		y_press = event->y / prefs.tile_height;
 		if (gi_game_running) {
 			gi_x_click = x_press;
 			gi_y_click = y_press;
 			gi_gem_clicked = -1;
 			gi_dragging = -1;
+
+			if(click_sfx)
+				Sample_Play(click_sfx, 0, 0);						
+
 		} else {
 			on_new1_activate (NULL, NULL);
 		}
@@ -210,8 +162,8 @@ drawing_area_button_event_cb (GtkWidget * widget, GdkEventButton * event, gpoint
 	case GDK_BUTTON_RELEASE:
 		gi_dragging = 0;
 		gi_gem_dragged = 0;		
-		x_release = event->x / gi_tile_width;
-		y_release = event->y / gi_tile_height;
+		x_release = event->x / prefs.tile_width;
+		y_release = event->y / prefs.tile_height;
 		if( (x_release < 0) ||
 			(x_release > BOARD_WIDTH - 1) ||
 			(y_release < 0) ||
@@ -239,8 +191,8 @@ gboolean
 drawing_area_motion_event_cb (GtkWidget * widget, GdkEventMotion * event, gpointer user_data)
 {
 	if (gi_dragging) {
-		gi_x_drag = event->x / gi_tile_width;
-		gi_y_drag = event->y / gi_tile_height;
+		gi_x_drag = event->x / prefs.tile_width;
+		gi_y_drag = event->y / prefs.tile_height;
 		gi_gem_dragged = -1;
 	}
 	
@@ -271,6 +223,7 @@ on_highscoresDialog_delete_event (GtkWidget * widget, GdkEvent * event, gpointer
 void
 on_closebutton1_clicked (GtkButton * button, gpointer user_data)
 {
+	save_preferences();
 	gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
 }
 
@@ -285,7 +238,7 @@ on_easyRadiobutton_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
 	if (gtk_toggle_button_get_active (togglebutton))
 	{
-		gweled_in_timer_mode = FALSE;
+		prefs.timer_mode = FALSE;
 		if (gi_game_running) {
 			sge_destroy_all_objects ();
 			gweled_draw_board ();
@@ -299,7 +252,7 @@ on_hardRadiobutton_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
 	if (gtk_toggle_button_get_active (togglebutton))
 	{
-		gweled_in_timer_mode = TRUE;
+		prefs.timer_mode = TRUE;
 		if (gi_game_running) {
 			sge_destroy_all_objects ();
 			gweled_draw_board ();
@@ -323,22 +276,21 @@ void
 on_smallRadiobutton_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
 	if (gtk_toggle_button_get_active (togglebutton)) {
-		gi_tile_width = 32;
-		gi_tile_height = 32;
+		prefs.tile_width = 32;
+		prefs.tile_height = 32;
 
 		gweled_reload_pixmaps ();
 
 		gtk_widget_set_size_request (GTK_WIDGET (g_drawing_area),
-					     BOARD_WIDTH * gi_tile_width,
-					     BOARD_HEIGHT *
-					     gi_tile_height);
+					     BOARD_WIDTH * prefs.tile_width,
+					     BOARD_HEIGHT * prefs.tile_height);
 		g_object_unref (G_OBJECT (g_buffer_pixmap));
 		g_buffer_pixmap = gdk_pixmap_new (g_drawing_area->window,
-				    BOARD_WIDTH * gi_tile_width,
-				    BOARD_HEIGHT * gi_tile_height, -1);
+				    BOARD_WIDTH * prefs.tile_width,
+				    BOARD_HEIGHT * prefs.tile_height, -1);
 		sge_set_drawing_area (g_drawing_area, g_buffer_pixmap,
-				      BOARD_WIDTH * gi_tile_width,
-				      BOARD_HEIGHT * gi_tile_height);
+				      BOARD_WIDTH * prefs.tile_width,
+				      BOARD_HEIGHT * prefs.tile_height);
 
 		gweled_reload_pixmaps ();
 	}
@@ -348,23 +300,22 @@ void
 on_mediumRadiobutton_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
 	if (gtk_toggle_button_get_active (togglebutton)) {
-		gi_tile_width = 48;
-		gi_tile_height = 48;
+		prefs.tile_width = 48;
+		prefs.tile_height = 48;
 
 		gweled_reload_pixmaps ();
 
 		gtk_widget_set_size_request (GTK_WIDGET (g_drawing_area),
-					     BOARD_WIDTH * gi_tile_width,
-					     BOARD_HEIGHT *
-					     gi_tile_height);
+					     BOARD_WIDTH * prefs.tile_width,
+					     BOARD_HEIGHT * prefs.tile_height);
 		g_object_unref (G_OBJECT (g_buffer_pixmap));
 		g_buffer_pixmap = gdk_pixmap_new (g_drawing_area->window,
-				    BOARD_WIDTH * gi_tile_width,
-				    BOARD_HEIGHT * gi_tile_height, -1);
+				    BOARD_WIDTH * prefs.tile_width,
+				    BOARD_HEIGHT * prefs.tile_height, -1);
 
 		sge_set_drawing_area (g_drawing_area, g_buffer_pixmap,
-				      BOARD_WIDTH * gi_tile_width,
-				      BOARD_HEIGHT * gi_tile_height);
+				      BOARD_WIDTH * prefs.tile_width,
+				      BOARD_HEIGHT * prefs.tile_height);
 	}
 }
 
@@ -372,22 +323,21 @@ void
 on_largeRadiobutton_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
 	if (gtk_toggle_button_get_active (togglebutton)) {
-		gi_tile_width = 64;
-		gi_tile_height = 64;
+		prefs.tile_width = 64;
+		prefs.tile_height = 64;
 
 		gweled_reload_pixmaps ();
 
 		gtk_widget_set_size_request (GTK_WIDGET (g_drawing_area),
-					     BOARD_WIDTH * gi_tile_width,
-					     BOARD_HEIGHT *
-					     gi_tile_height);
+					     BOARD_WIDTH * prefs.tile_width,
+					     BOARD_HEIGHT * prefs.tile_height);
 		g_object_unref (G_OBJECT (g_buffer_pixmap));
 		g_buffer_pixmap = gdk_pixmap_new (g_drawing_area->window,
-				    BOARD_WIDTH * gi_tile_width,
-				    BOARD_HEIGHT * gi_tile_height, -1);
+				    BOARD_WIDTH * prefs.tile_width,
+				    BOARD_HEIGHT * prefs.tile_height, -1);
 
 		sge_set_drawing_area (g_drawing_area, g_buffer_pixmap,
-				      BOARD_WIDTH * gi_tile_width,
-				      BOARD_HEIGHT * gi_tile_height);
+				      BOARD_WIDTH * prefs.tile_width,
+				      BOARD_HEIGHT * prefs.tile_height);
 	}
 }
