@@ -1,13 +1,11 @@
-// CONSTANTS
+#ifndef _GRAPHIC_ENGINE_H
+#define _GRAPHIC_ENGINE_H
+
 #define BOARD_WIDTH   8
 #define BOARD_HEIGHT  8
 #define FONT_WIDTH    24
 #define FONT_HEIGHT   20
 
-// EXPORTS
-extern gint gi_tile_width;
-extern gint gi_tile_height;
-
 // FUNCTIONS
 
 void gweled_draw_message(gchar *in_message);
@@ -16,3 +14,4 @@ void gweled_draw_message_at(gchar *in_message, gint msg_x, gint msg_y);
 int gweled_gems_fall_into_place(void);
 T_SGEObject *gweled_draw_character(int x, int y, int layer, char character);
 
+#endif
