@@ -1,3 +1,5 @@
+#ifndef _SGE_CORE_H_
+#define _SGE_CORE_H_
 
 typedef struct s_sge_object
 {
@@ -9,13 +11,13 @@ typedef struct s_sge_object
 	gdouble ay;
 	gint dest_x;
 	gint dest_y;
-  gint width;
-  gint height;
+	gint width;
+	gint height;
 	gint layer;
 	gint lifetime;
-  gint needs_drawing;
-  gint pixbuf_id;	
-  GdkPixmap *pre_rendered;
+	gint needs_drawing;
+	gint pixbuf_id;	
+	GdkPixmap *pre_rendered;
 	gint (*stop_condition)(struct s_sge_object *);
 	GFunc stop_callback;
 }T_SGEObject;
@@ -41,3 +43,4 @@ gboolean sge_objects_are_moving_on_layer(int layer);
 
 void sge_invalidate_layer(int layer);
 
+#endif
